@@ -5,6 +5,8 @@ Tests for `attr._make`.
 from __future__ import absolute_import, division, print_function
 
 import pytest
+from hypothesis import given
+from hypothesis.strategies import booleans
 
 from . import simple_attr
 from attr import _config
@@ -96,7 +98,7 @@ class TestTransformAttrs(object):
             "No mandatory attributes allowed after an attribute with a "
             "default value or factory.  Attribute in question: Attribute"
             "(name='y', default=NOTHING, validator=None, repr=True, "
-            "cmp=True, hash=True, init=True)",
+            "cmp=True, hash=True, init=True, convert=None)",
         ) == e.value.args
 
     def test_these(self):
@@ -246,26 +248,28 @@ class TestAttributes(object):
         assert sentinel == getattr(C, method_name)
 
     @pytest.mark.skipif(not PY3, reason="__qualname__ is PY3-only.")
-    def test_repr_qualname(self):
+    @given(slots_outer=booleans(), slots_inner=booleans())
+    def test_repr_qualname(self, slots_outer, slots_inner):
         """
         On Python 3, the name in repr is the __qualname__.
         """
-        @attributes
+        @attributes(slots=slots_outer)
         class C(object):
-            @attributes
+            @attributes(slots=slots_inner)
             class D(object):
                 pass
 
         assert "C.D()" == repr(C.D())
         assert "GC.D()" == repr(GC.D())
 
-    def test_repr_fake_qualname(self):
+    @given(slots_outer=booleans(), slots_inner=booleans())
+    def test_repr_fake_qualname(self, slots_outer, slots_inner):
         """
         Setting repr_ns overrides a potentially guessed namespace.
         """
-        @attributes
+        @attributes(slots=slots_outer)
         class C(object):
-            @attributes(repr_ns="C")
+            @attributes(repr_ns="C", slots=slots_inner)
             class D(object):
                 pass
         assert "C.D()" == repr(C.D())
@@ -297,7 +301,7 @@ class TestAttribute(object):
         with pytest.raises(TypeError) as e:
             Attribute(name="foo", default=NOTHING,
                       factory=NOTHING, validator=None,
-                      repr=True, cmp=True, hash=True, init=True)
+                      repr=True, cmp=True, hash=True, init=True, convert=None)
         assert ("Too many arguments.",) == e.value.args
 
 
@@ -392,6 +396,34 @@ class TestFields(object):
                    in zip(fields(C), C.__attrs_attrs__))
 
 
+class TestConvert(object):
+    """
+    Tests for attribute conversion.
+    """
+    def test_convert(self):
+        """
+        Return value of convert is used as the attribute's value.
+        """
+        C = make_class("C", {"x": attr(convert=lambda v: v + 1),
+                             "y": attr()})
+        c = C(1, 2)
+        assert c.x == 2
+        assert c.y == 2
+
+    def test_convert_before_validate(self):
+        """
+        Validation happens after conversion.
+        """
+        def validator(inst, attr, val):
+            raise RuntimeError("foo")
+        C = make_class(
+            "C",
+            {"x": attr(validator=validator, convert=lambda v: 1 / 0),
+             "y": attr()})
+        with pytest.raises(ZeroDivisionError):
+            C(1, 2)
+
+
 class TestValidate(object):
     """
     Tests for `validate`.
